@@ -28,6 +28,8 @@ reload( utils )
 
 from xceptions import baseException
 
+from fcc_utils2 import tupsort
+
 
 #driving_model_ensemble_member = <CMIP5Ensemble_member>
 #rcm_version_id = <RCMVersionID>                     
@@ -36,11 +38,15 @@ class dummy(object):
    pass
 
 pathTmplDict = { 'CORDEX':'%(project)s/%(product)s/%(domain)s/%(institute)s/%(driving_model)s/%(experiment)s/%(ensemble)s/%(model)s/%(model_version)s/%(frequency)s/%(variable)s/files/%%(version)s/',   \
-                 'SPECS':'%(project)s/%(product)s/%(institute)s/%(model)s/%(experiment)s_%(series)s/%(start_date)s/%(frequency)s/%(realm)s/%(table)s/%(variable)s/%(ensemble)s/files/%%(version)s/', \
+                 'SPECS':'%(project)s/%(product)s/%(institute)s/%(model)s/%(experiment)s/%(start_date)s/%(frequency)s/%(realm)s/%(table)s/%(variable)s/%(ensemble)s/files/%%(version)s/', \
                  'CMIP5':'%(project)s/%(product)s/%(institute)s/%(model)s/%(experiment)s/%(frequency)s/%(realm)s/%(table)s/%(ensemble)s/files/%%(version)s/%(variable)s/', \
                  '__def__':'%(project)s/%(product)s/%(institute)s/%(model)s/%(experiment)s/%(frequency)s/%(realm)s/%(variable)s/%(ensemble)s/files/%%(version)s/', \
                }
 
+## Core DRS: list of vocab names
+## Path template: -- current version puts upper case in "project"
+## Dataset template:  
+
 class recorder(object):
 
   def __init__(self,project,fileName,type='map',dummy=False):
@@ -50,6 +56,7 @@ class recorder(object):
     self.pathTmpl = '%(project)s/%(product)s/%(domain)s/%(institute)s/%(driving_model)s/%(experiment)s/%(ensemble)s/%(model)s/%(model_version)s/%(frequency)s/%(variable)s/files/%%(version)s/'
     self.pathTmpl = pathTmplDict.get(project,pathTmplDict['__def__'])
     self.records = {}
+    self.tidtupl = []
 
   def open(self):
     if self.type == 'map':
@@ -75,11 +82,13 @@ class recorder(object):
       fdate = "na"
       sz = 0
     record = '%s | OK | %s | modTime = %s | target = %s ' % (fpath,sz,fdate,tpath)
+    fn = string.split( fpath, '/' )[-1]
     for k in ['creation_date','tracking_id']:
       if k in drs.keys():
         record += ' | %s = %s' % (k,drs[k])
+        if k == 'tracking_id':
+          self.tidtupl.append( (fn,drs[k]) )
 
-    fn = string.split( fpath, '/' )[-1]
     self.records[fn] = record
   
   def modify(self,fn,msg):
@@ -91,6 +100,27 @@ class recorder(object):
     ##print '--> ',s
     self.records[fn] = s
 
+  def checktids(self):
+## sort by tracking id
+    if len( self.tidtupl ) == 1:
+      return
+    self.tidtupl.sort( cmp=tupsort(k=1).cmp )
+    nd = 0
+    fnl = []
+    for k in range(len(self.tidtupl)-1):
+      if self.tidtupl[k][1] == self.tidtupl[k+1][1]:
+        print 'Duplicate tracking_id: %s, %s:: %s' % (self.tidtupl[k][0],self.tidtupl[k+1][0],self.tidtupl[k][1])
+        nd += 1
+        if len(fnl) == 0 or fnl[-1] != self.tidtupl[k][0]:
+          fnl.append( self.tidtupl[k][0])
+        fnl.append( self.tidtupl[k+1][0])
+    if nd == 0:
+      print 'No duplicate tracking ids found in %s files' % len(self.tidtupl)
+    else:
+      print '%s duplicate tracking ids' % nd
+      for f in fnl:
+        self.modify( f, 'ERROR: duplicate tid' )
+
   def dumpAll(self,safe=True):
     keys = self.records.keys()
     keys.sort()
@@ -230,12 +260,16 @@ class c4_init(object):
     if self.summarymode:
       return
 
+    self.forceNetcdfLib = None
+    fltype = None
     argu = []
     while len(args) > 0:
       next = args.pop(0)
       if next == '-f':
         flist = [args.pop(0),]
         self.logByFile = False
+        fltype = '-f'
+        self.source = flist[0]
       elif next == '--log':
         x = args.pop(0)
         assert x in ['single','multi','s','m'], 'unrecognised logging option (--log): %s' % (x)
@@ -243,6 +277,14 @@ class c4_init(object):
            forceLogOrg = 'multi'
         elif x in ['single','s']:
            forceLogOrg = 'single'
+      elif next == '--force-ncq':
+        self.forceNetcdfLib = 'ncq3'
+      elif next == '--force-cdms2':
+        self.forceNetcdfLib = 'cdms2'
+      elif next == '--force-pync4':
+        self.forceNetcdfLib = 'netCDF4'
+      elif next == '--force-scientific':
+        self.forceNetcdfLib = 'Scientific'
       elif next == '--flfmode':
         lfmk = args.pop(0)
         assert lfmk in ['a','n','np','w','wo'], 'Unrecognised file logfile mode (--flfmode): %s' % lfmk
@@ -254,6 +296,7 @@ class c4_init(object):
       elif next == '-d':
         fdir = args.pop(0)
         flist = glob.glob( '%s/*.nc' % fdir  )
+        self.source = '%s/*.nc' % fdir
       elif next == '-D':
         flist  = []
         fdir = args.pop(0)
@@ -262,6 +305,7 @@ class c4_init(object):
             fpath = '%s/%s' % (root,f)
             if (os.path.isfile( fpath ) or os.path.islink( fpath )) and f[-3:] == '.nc':
               flist.append( fpath )
+        self.source = '%s/.....' % fdir
       elif next == '-R':
         self.recordFile = args.pop(0)
       elif next == '--ld':
@@ -281,7 +325,7 @@ class c4_init(object):
       print 'Unused arguments: ', argu
       self.commandHints( argsIn )
 
-    if self.project == 'CMIP5':
+    if self.project == 'CMIP5' and fltype != '-f':
       fl0 = []
       for f in flist:
         if string.find( f, '/latest/' ) != -1:
@@ -427,7 +471,7 @@ class main(object):
     if (ncLib == None) and (not isDummy):
        raise baseException( 'Cannot proceed with non-dummy [%s] project without a netcdf API' % (c4i.project) )
     pcfg = config.projectConfig( c4i.project )
-    ncReader = fileMetadata(dummy=isDummy, attributeMappingsLog=c4i.attributeMappingsLog)
+    ncReader = fileMetadata(dummy=isDummy, attributeMappingsLog=c4i.attributeMappingsLog,forceLib=c4i.forceNetcdfLib)
     self.cc = checker(pcfg, c4i.project, ncReader,abortMessageCount=abortMessageCount)
     rec = recorder( c4i.project, c4i.recordFile, dummy=isDummy )
     if monitorFileHandles:
@@ -437,6 +481,7 @@ class main(object):
 
     cal = None
     c4i.logger.info( 'Starting batch -- number of file: %s' % (len(c4i.flist)) )
+    c4i.logger.info( 'Source: %s' % c4i.source )
     if len( c4i.errs ) > 0:
       for i in range(0,len( c4i.errs ), 2 ):
         c4i.logger.info( c4i.errs[i] )
@@ -449,7 +494,11 @@ class main(object):
 
     fileLogOpen = False
     self.resList =  []
+    stdoutsum = 2000
+    npass = 0
+    kf = 0
     for f in c4i.flist:
+      kf += 1
       rv = False
       ec = None
       if monitorFileHandles:
@@ -482,6 +531,8 @@ class main(object):
           cal = self.cc.calendar
           ec = self.cc.errorCount
         rv =  ec == 0
+        if rv:
+          npass += 1
         self.resList.append( (rv,ec) )
 
         if c4i.logByFile:
@@ -512,6 +563,8 @@ class main(object):
         rec.addErr( f, 'ERROR: Exception' )
         if not c4i.holdExceptions:
           raise
+      if stdoutsum > 0 and kf%stdoutsum == 0:
+         print '%s files checked; %s passed this round' % (kf,npass)
       if monitorFileHandles:
         nofhEnd = self.monitor.get_open_fds()
         if nofhEnd > nofhStart:
@@ -537,6 +590,8 @@ class main(object):
         if ll[i] != ll[i-1]:
           oo.write( ll[i] + '\n' )
       oo.close()
+    if c4i.project in ['SPECS','CCMI','CMIP5']:
+      rec.checktids()
     rec.dumpAll()
     if printInfo:
       print 'Error count %s' % ecount
